import argparse
from pvcpie.kubernetesUtils import KubernetesUtils
from prettytable import PrettyTable


parser = argparse.ArgumentParser(
    prog="Pvcpie",
    usage="%(prog)s [options]",
    description="Provide input flags as per your requirement",
)

parser.add_argument(
    "-i",
    "--incluster",
    action="store_true",
    help="Use this flag if you want to load in cluster config intead of kube config",
)

parser.add_argument(
    "--api",
    action="store_true",
    help="Use to run in CLI mode",
)


args = parser.parse_args()


def bytes_to_readable(byte_value):
    if byte_value < 1024:
        return f"{byte_value} bytes"
    elif byte_value < 1024 * 1024:
        return f"{byte_value / 1024:.2f} KB"
    elif byte_value < 1024 * 1024 * 1024:
        return f"{byte_value / (1024 * 1024):.2f} MB"
    else:
        return f"{byte_value / (1024 * 1024 * 1024):.2f} GB"


def run_cli_mode(kts):
    nodes = kts.list_all_nodes()
    output = PrettyTable(
        [
            "PVC Name",
            "Namespace",
            "AvailableBytes",
            "CapacityBytes",
            "UsedBytes",
            "% Used",
        ]
    )

    for node in nodes:
        res = kts.get_node_summary(node)
        volumes = kts.get_pvc_summary(res)

        for vol in volumes:
            output.add_row(
                [
                    vol["pvc_name"],
                    vol["namespace"],
                    bytes_to_readable(vol["availableBytes"]),
                    bytes_to_readable(vol["capacityBytes"]),
                    bytes_to_readable(vol["usedBytes"]),
                    f'{(vol["usedBytes"] * 100) / (vol["capacityBytes"]):.2f}',
                ]
            )
    print(output)


def run_api_mode(kts):
    nodes = kts.list_all_nodes()
    data = []

    for node in nodes:
        res = kts.get_node_summary(node)
        volumes = kts.get_pvc_summary(res)

        for vol in volumes:
            data.append(
                {
                    "PVC Name": vol["pvc_name"],
                    "Namespace": vol["namespace"],
                    "AvailableBytes": bytes_to_readable(vol["availableBytes"]),
                    "CapacityBytes": bytes_to_readable(vol["capacityBytes"]),
                    "UsedBytes": bytes_to_readable(vol["usedBytes"]),
                    "% Used": f'{(vol["usedBytes"] * 100) / (vol["capacityBytes"]):.2f}',
                }
            )
    print(data)
    return data


def main():
    kts = KubernetesUtils(incluster_config=args.incluster)

    if args.api:
        run_api_mode(kts)
    else:
        run_cli_mode(kts)



import json
from kubernetes import client, config


class KubernetesUtils:
    def __init__(self, incluster_config=False):
        try:
            if incluster_config:
                config.load_incluster_config()
            else:
                config.load_kube_config()
        except Exception as e:
            print("Unable to load config")

        self.client = client.CoreV1Api()

    def get_namespaces(self):
        try:
            namespaces = self.client.list_namespace()
            res = [ns.metadata.name for ns in namespaces.items]
            return res
        except Exception as e:
            print(f"Unable to get namespaces. Error: {e}")

    def get_namespace_pods(self, namespace):
        try:
            res = self.client.list_namespaced_pod(namespace)
            pods = [pod.metadata.name for pod in res.items]
            return pods
        except Exception as e:
            print(f"Unable to get namespaces. Error: {e}")

    def get_pod_node_name(self, pod_name, namespace):
        try:
            pod = self.client.read_namespaced_pod(pod_name, namespace)
            return pod.spec.node_name
        except Exception as e:
            print(f"Unable to get namespaces. Error: {e}")

    def list_all_nodes(self):
        try:
            res = self.client.list_node(pretty=True)
            nodes = [node.metadata.name for node in res.items]
            return nodes
        except Exception as e:
            print(f"Unable to get nodes. Error: {e}")

    def get_node_summary(self, node_name):
        try:
            res = self.client.connect_get_node_proxy_with_path(
                node_name, path="/stats/summary"
            )
            res = res.replace("'", '"')
            res = json.loads(res)
            return res
        except Exception as e:
            print(f"Unable to get node summary. Error: {e}")

    def get_pvc_summary(self, node_summary):
        try:
            pods = node_summary["pods"]
            pod_volumes = [pod["volume"] for pod in pods if "volume" in pod]

            pvc_volumes = []
            for volumes in pod_volumes:
                for vol in volumes:
                    if "pvcRef" in vol:
                        pvc_volumes.append(
                            {
                                "pvc_name": vol["pvcRef"]["name"],
                                "namespace": vol["pvcRef"]["namespace"],
                                "availableBytes": vol["availableBytes"],
                                "capacityBytes": vol["capacityBytes"],
                                "usedBytes": vol["usedBytes"],
                            }
                        )
            return pvc_volumes
        except Exception as e:
            print(f"Error parsing summary {e}")
